#include <iostream>
#include <stdlib.h> //Needed for "exit" function

// Include OpenGL header files, so that we can use OpenGL
#ifdef __APPLE__
#include <OpenGL/OpenGL.h>
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <math.h>

#include "flockobj.h"

#define DEFAULT_SIZE 3
#define MAX_SPEED 0.30

using namespace std;

// constructor
flockobj::flockobj() {
    x = 0; y = 0; z = 0;
    dx = 0; dy = 0; dz = 0;
    xrot = 0; yrot = 0;
    size = DEFAULT_SIZE;
}

flockobj::flockobj(float a, float b, float c, float s) {
    x = a;
    y = b;
    z = c;
    size = s;
    yrot = 0;
    xrot = 0;
    dx = 0;
    dy = 0;
    dz = 0;
}

// set the location of this object
void flockobj::set_loc(float a, float b, float c) {
    x = a;
    y = b;
    z = c;
}

// set the speed of the object
void flockobj::set_speed(float a, float b, float c) {
    dx = a;
    dy = b;
    dz = c;
	update_rots();
}

// apply a scaled acceleration to the object
void flockobj::accelerate(float ddx, float ddy, float ddz, float scaling) {
	dx += ddx * scaling;
	dy += ddy * scaling;
	dz += ddz * scaling;
	update_rots();
}

// limit the speed of the object based on the constant MAX_SPEED
void flockobj::apply_topspeed() {
	float speed_scale = sqrt(pow(dx,2)+pow(dy,2)+pow(dz,2)) / MAX_SPEED;
	if (speed_scale > 1) {
		dx = dx / speed_scale;
		dy = dy / speed_scale;
		dz = dz / speed_scale;
	}
}

// set the heading and attitude of the object
void flockobj::update_rots() {
	// set y rotation (heading)
	yrot = atan2(dx, dz) * (float)180 / 3.1415;

    // set x rotation (attitude)
    xrot = -atan2(dy, sqrt(pow(dx,2) + pow(dz,2))) * (float)180 / 3.1415;
}

// draw the object as a sphere
void flockobj::draw() {
    glPushMatrix();
    glTranslatef(x, y, z);
    glutSolidSphere(size, 32, 32);
    glPopMatrix();
}

// move the object based on its location and velocity
void flockobj::updateloc() {
    x += dx;
    y += dy;
    z += dz;
}
