#include <iostream>
#include <stdlib.h> //Needed for "exit" function
#include <cstdlib>  // Needed for "rand"
#include <ctime>

// Include OpenGL header files, so that we can use OpenGL
#ifdef __APPLE__
#include <OpenGL/OpenGL.h>
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <math.h>

#include "flock.h"

#ifndef TARGET_SHAPE
#define TARGET_SHAPE
#include "targetshape.h"
#endif

#define WIDTH 800
#define HEIGHT 600

// input buffer to collect typing
#define BUFFERSIZE 32

// features related to the easter egg
#define MAX_COUNTER 200
#define NORMAL 0
#define HEART 1

using namespace std;

// global variables
flock myflock;
targetshape myheart;

// the password is "fly, my pretties"
int passwordtimesten[] = {1020, 1080, 1210, 440, 320, 1090, 1210, 
						320, 1120, 1140, 1010, 1160, 1160, 1050, 
						1010, 1150, 0};

char buffer[BUFFERSIZE];
int counter = 0;
bool mode = NORMAL; 

//Called when a key is pressed
void handleKeypress(unsigned char key, int x, int y)  {
    if (key == 27) exit(0); //Exit the program
	else if (mode == HEART){
		mode = NORMAL;
		for(int i=0; i<BUFFERSIZE; i++) buffer[i] = 0;
		buffer[BUFFERSIZE-1] = key;
	}
	else { // if not, stick it into the buffer, shifting as necessary
		for(int i=0; i<BUFFERSIZE-1; i++) buffer[i] = buffer[i+1];
		buffer[BUFFERSIZE-1] = key;

		// now, check for the password
		for(int i=0; i<BUFFERSIZE; i++) {
			for(int j=0; passwordtimesten[j]; j++) {
				if (buffer[i+j] != (char)(passwordtimesten[j]/10)) break;
				if (!passwordtimesten[j+1]) mode = HEART;
			}
		}
	}
}

// Initializes 3D rendering
void initRendering() {
    glEnable(GL_DEPTH_TEST);
}

void handleResize(int w, int h) {
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0,    // camera angle
            (double)w / (double)h, // width-to-height ratio
            1.0, // near z clipping coordinate
            200.0); // far z clipping coordinate
}

void handleMouse(int x, int y) {
	myflock.set_target( (x-WIDTH/2)/7.2,
						-(y-HEIGHT/2)/7.2,
						-100);
}

// draw the scene
void drawScene() {
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW); // switch to drawing perspective
    glLoadIdentity(); // reset perspective
    
    float temp;
    
    if (mode == HEART) counter += (counter < MAX_COUNTER);
    else {
		myflock.drawtarget();
		counter -= (counter > 0);
    }
    temp = 0.8*(float)counter/(float)MAX_COUNTER;
    myflock.draw(1, 1-temp, 1-temp);

	
    glutSwapBuffers();
}

// the functions to update the scene
void update(int value) {
	glutTimerFunc(25, update, 0);
	
	if (mode == HEART) myflock.makeshape(myheart);
	else myflock.update();
    
    glutPostRedisplay();
}

int main(int argc, char** argv) {
	// seed the random number generator
	srand((unsigned)time(0));
	
	// initialize glut
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize(WIDTH, HEIGHT);
    glutCreateWindow("flock");
    initRendering();
	
	// initialize various things
	myflock.initialize();
	myheart.initialize_heart(myflock.get_flocksize());
	
	// set up the functions
    glutDisplayFunc(drawScene);
    glutKeyboardFunc(handleKeypress);
    glutReshapeFunc(handleResize);
    glutPassiveMotionFunc(handleMouse);
    
    // start the looping
    glutTimerFunc(25, update, 0);

    glutMainLoop();
    return 0;
}
