#include <math.h>
#include <iostream>

#ifdef __APPLE__
#include <OpenGL/OpenGL.h>
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "targetshape.h"

// initialize the vertices of a heart shape
void targetshape::initialize_heart(int np) {
	vertices[0][0] = 0;
	vertices[0][1] = -35;
	vertices[0][2] = -100;
	
	vertices[1][0] = -40;
	vertices[1][1] = 10;
	vertices[1][2] = -100;
	
	vertices[2][0] = -35;
	vertices[2][1] = 30;
	vertices[2][2] = -100;
	
	vertices[3][0] = -15;
	vertices[3][1] = 35;
	vertices[3][2] = -100;
	
	// top of the heart
	vertices[4][0] = 0;
	vertices[4][1] = 20;
	vertices[4][2] = -100;
	
	vertices[5][0] = 15;
	vertices[5][1] = 35;
	vertices[5][2] = -100;
	
	vertices[6][0] = 35;
	vertices[6][1] = 30;
	vertices[6][2] = -100;
	
	vertices[7][0] = 40;
	vertices[7][1] = 10;
	vertices[7][2] = -100;
	
	vertices[8][0] = 0;
	vertices[8][1] = -35;
	vertices[8][2] = -100;
	
	numvertices = 9;
	numpoints = np;
	
	// and now, generate the set of points that interpolate between vertices
	generate_points();
}

// subroutine to interpolate a set of points between vertices
void targetshape::generate_points() {
	
	float total_dist = 0;
	
	// calculate total length of the shape
	for(int i=0; i<numvertices-1; i++) {
		total_dist += sqrt(
						pow(vertices[i][0]-vertices[i+1][0],2) +
						pow(vertices[i][1]-vertices[i+1][1],2) +
						pow(vertices[i][2]-vertices[i+1][2],2));
	}
	
	// stick the points on the shape
	int assigned_points = 0;
	float frac_dist = 0;
	int counter = 0;

	int points_on_segment;
	float movement_vector[3];
	float current_point[3];

	for(int i=0; i<numvertices-1; i++) {
		frac_dist += sqrt(
						pow(vertices[i][0]-vertices[i+1][0],2) +
						pow(vertices[i][1]-vertices[i+1][1],2) +
						pow(vertices[i][2]-vertices[i+1][2],2)) / total_dist;
		
		// figure out how many of the points should be on this segment
		points_on_segment = int(frac_dist*numpoints + 0.5) - assigned_points;
		assigned_points += points_on_segment;

		// calculate the direction of this line segment
		for(int j=0; j<3; j++) {
			current_point[j] = vertices[i][j];	
			movement_vector[j] = 
				(vertices[i+1][j] - vertices[i][j]) / points_on_segment;
		}

		// store the points
		for(int j=0; j<points_on_segment; j++) {
			for(int k=0; k<3; k++) {
				points[counter][k] = current_point[k];
				current_point[k] += movement_vector[k];
			}
			counter++;
		}	
	}
}

// draws out the set of target points as spheres
void targetshape::draw() {
	for(int i=0; i<numpoints; i++) {
		glPushMatrix();
		glTranslatef(points[i][0], points[i][1], points[i][2]);
		glColor3f(1, 1, 1);
		glutSolidSphere(0.5,16, 16);
		glPopMatrix();
	}
}

// set the value of the ith target point
void targetshape::get_point_loc(int i, float output[]) {
	output[0] = points[i][0];
	output[1] = points[i][1];
	output[2] = points[i][2];
}